@@ -27,6 +27,7 @@ class TypeMap(dict):
     def __init__(self):
         self.entries = {}
         self.types = set()
+        self.__functions__ = set()
 
     def register(self, obj_t, handler):
         """Register a handler for the given object type."""
@@ -34,6 +35,7 @@ class TypeMap(dict):
         self.types.add(obj_t)
         s = self.entries.setdefault(obj_t, set())
         s.add(handler)
+        self.__functions__.add(handler)
 
     def __missing__(self, obj_t):
         """Get the handler for the given type.
@@ -79,6 +81,7 @@ class MultiTypeMap(dict):
         self.empty = MISSING
         self.transform = type
         self.key_error = key_error
+        self.__functions__ = set()
 
     def register(self, obj_t_tup, nargs, handler):
         """Register a handler for a tuple of argument types.
@@ -108,6 +111,8 @@ class MultiTypeMap(dict):
             tm = self.maps.setdefault(-1, TypeMap())
             tm.register(object, entry)
 
+        self.__functions__.add(handler)
+
     def __missing__(self, obj_t_tup):
         specificities = {}
         candidates = None
@@ -242,6 +247,12 @@ class _Ovld:
         name: Optional name for the Ovld. If not provided, it will be
             gotten automatically from the first registered function or
             dispatch.
+        mapper: Class implementing a mapping interface from a tuple of
+            types to a handler (default: MultiTypeMap).
+        linkback: Whether to keep a pointer in the parent mixins to this
+            ovld so that updates can be propagated. (default: False)
+        allow_replacement: Allow replacing a method by another with the
+            same signature. (default: True)
     """
 
     def __init__(
@@ -255,15 +266,20 @@ class _Ovld:
         bootstrap=None,
         name=None,
         mapper=MultiTypeMap,
+        linkback=False,
+        allow_replacement=True,
     ):
         """Initialize an Ovld."""
         self._compiled = False
         self._dispatch = dispatch
         self.maindoc = None
         self.mapper = mapper
+        self.linkback = linkback
+        self.children = []
         self.type_error = type_error
         self.initial_state = initial_state
         self.postprocess = postprocess
+        self.allow_replacement = allow_replacement
         self.bootstrap_class = OvldCall
         if self.initial_state or self.postprocess:
             assert bootstrap is not False
@@ -300,6 +316,8 @@ class _Ovld:
     def add_mixins(self, *mixins):
         self._attempt_modify()
         for mixin in mixins:
+            if self.linkback:
+                mixin.children.append(self)
             assert mixin.bootstrap is not None
             if self.bootstrap is None:
                 self.bootstrap = mixin.bootstrap
@@ -410,7 +428,8 @@ class _Ovld:
         modification.
         """
         for mixin in self.mixins:
-            mixin.lock()
+            if self not in mixin.children:
+                mixin.lock()
         self._compiled = True
         self.map = self.mapper(key_error=self._key_error)
 
@@ -455,10 +474,23 @@ class _Ovld:
         """Find the correct method to call for the given arguments."""
         return self.map[tuple(map(self.map.transform, args))]
 
-    def register_signature(self, key, fn):
+    def register_signature(self, key, orig_fn):
         """Register a function for the given signature."""
+
+        def conform(new_fn):
+            if new_fn is None:
+                self.unregister(orig_fn)
+            elif inspect.signature(orig_fn) != inspect.signature(new_fn):
+                self.unregister(orig_fn)
+                self.register(new_fn)
+            else:
+                orig_fn.__code__ = fn.__code__ = new_fn.__code__
+
         sig, min, max, vararg = key
-        fn = rename_function(fn, f"{self.__name__}[{self._sig_string(sig)}]")
+        fn = rename_function(
+            orig_fn, f"{self.__name__}[{self._sig_string(sig)}]"
+        )
+        fn.__conform__ = conform
         self.map.register(sig, (min, max, vararg), fn)
         return self
 
@@ -493,15 +525,34 @@ class _Ovld:
             t if isinstance(t, tuple) else (t,) for t in typelist
         )
         for tl in itertools.product(*typelist_tups):
-            self._defns[tuple(tl), req_pos, max_pos, bool(argspec.varargs)] = fn
+            sig = (tuple(tl), req_pos, max_pos, bool(argspec.varargs))
+            if not self.allow_replacement and sig in self._defns:
+                raise TypeError(f"There is already a method for {tl}")
+            self._defns[(*sig,)] = fn
 
         self._make_signature()
-        if self._compiled:
-            self.compile()
+        self._update()
         return self
 
+    def unregister(self, fn):
+        """Unregister a function."""
+        self._attempt_modify()
+        self._defns = {sig: f for sig, f in self._defns.items() if f is not fn}
+        self._update()
+
+    def _update(self):
+        if self._compiled:
+            self.compile()
+        for child in self.children:
+            child._update()
+
     def copy(
-        self, dispatch=MISSING, initial_state=None, postprocess=None, mixins=[]
+        self,
+        dispatch=MISSING,
+        initial_state=None,
+        postprocess=None,
+        mixins=[],
+        linkback=False,
     ):
         """Create a copy of this Ovld.
 
@@ -514,29 +565,28 @@ class _Ovld:
             mixins=[self, *mixins],
             initial_state=initial_state or self.initial_state,
             postprocess=postprocess or self.postprocess,
+            linkback=linkback,
         )
 
     def variant(
-        self,
-        fn=None,
-        *,
-        dispatch=MISSING,
-        initial_state=None,
-        postprocess=None,
-        mixins=[],
+        self, fn=None, **kwargs,
     ):
         """Decorator to create a variant of this Ovld.
 
         New functions can be registered to the variant without affecting the
         original.
         """
-        ov = self.copy(dispatch, initial_state, postprocess, mixins)
+        ov = self.copy(**kwargs)
         if fn is None:
             return ov.register
         else:
             ov.register(fn)
             return ov
 
+    @_compile_first
+    def get_map(self):
+        return self.map
+
     @_compile_first
     def instantiate(self, **state):
         return self.ocls(map=self.map, state=state, bind_to=BOOTSTRAP)
@@ -549,7 +599,12 @@ class _Ovld:
             state = self.initial_state
         else:
             state = self.initial_state()
-        return self.ocls(map=self.map, state=state, bind_to=obj)
+        return self.ocls(
+            map=self.map,
+            state=state,
+            bind_to=obj,
+            super=self.mixins[0] if len(self.mixins) == 1 else None,
+        )
 
     @_compile_first
     def __getitem__(self, t):
@@ -591,13 +646,18 @@ class _Ovld:
     def __repr__(self):
         return f"<Ovld {self.name or hex(id(self))}>"
 
+    @property
+    def __functions__(self):
+        return self.map.__functions__
+
 
 class OvldCall:
     """Context for an Ovld call."""
 
-    def __init__(self, map, state, bind_to):
+    def __init__(self, map, state, bind_to, super=None):
         """Initialize an OvldCall."""
         self.map = map
+        self._parent = super
         if state is not None:
             self.__dict__.update(state)
         self.obj = self if bind_to is BOOTSTRAP else bind_to
@@ -608,10 +668,20 @@ class OvldCall:
             t = (t,)
         return self.map[t].__get__(self.obj)
 
+    def super(self, *args, **kwargs):
+        """Use the parent ovld's method for this call."""
+        pmap = self._parent.get_map()
+        method = pmap[tuple(map(pmap.transform, args))]
+        return method.__get__(self.obj)(*args, **kwargs)
+
     def resolve(self, *args):
         """Find the right method to call for the given arguments."""
         return self[tuple(map(self.map.transform, args))]
 
+    def call(self, *args):
+        """Call the right method for the given arguments."""
+        return self[tuple(map(self.map.transform, args))](*args)
+
     def with_state(self, **state):
         """Return a new OvldCall using the given state."""
         return type(self)(self.map, state, BOOTSTRAP)
@@ -657,6 +727,12 @@ class OvldMC(type):
     the same name and different type signatures.
     """
 
+    def create_subclass(cls, *bases, name=None):
+        """Create a new subclass with the given extra bases."""
+        name = name or "UnnamedOvld"
+        bases = (cls, *bases)
+        return type(cls)(name, bases, cls.__prepare__(name, bases))
+
     @classmethod
     def __prepare__(cls, name, bases):
         d = ovld_cls_dict()
@@ -668,9 +744,15 @@ class OvldMC(type):
         for name in names:
             values = [getattr(base, name, None) for base in bases]
             mixins = [v for v in values if isinstance(v, _Ovld)]
-            rest = [v for v in values if not isinstance(v, _Ovld)]
             if mixins:
                 o = mixins[0].copy(mixins=mixins[1:])
+                others = [
+                    v
+                    for v in values
+                    if v is not None and not isinstance(v, _Ovld)
+                ]
+                for other in others:
+                    o.register(other)
                 o.rename(name)
                 d[name] = o
 
@@ -679,7 +761,7 @@ class OvldMC(type):
 
 def _find_overload(fn, **kwargs):
     mod = __import__(fn.__module__, fromlist="_")
-    dispatch = getattr(mod, fn.__name__, None)
+    dispatch = getattr(mod, fn.__qualname__, None)
     if dispatch is None:
         dispatch = _fresh(_Ovld)(**kwargs)
     elif kwargs:  # pragma: no cover
@@ -717,6 +799,10 @@ def ovld(fn, **kwargs):
         name: Optional name for the Ovld. If not provided, it will be
             gotten automatically from the first registered function or
             dispatch.
+        mapper: Class implementing a mapping interface from a tuple of
+            types to a handler (default: MultiTypeMap).
+        linkback: Whether to keep a pointer in the parent mixins to this
+            ovld so that updates can be propagated. (default: False)
     """
     dispatch = _find_overload(fn, **kwargs)
     return dispatch.register(fn)
@@ -748,6 +834,10 @@ def ovld_dispatch(dispatch, **kwargs):
         name: Optional name for the Ovld. If not provided, it will be
             gotten automatically from the first registered function or
             dispatch.
+        mapper: Class implementing a mapping interface from a tuple of
+            types to a handler (default: MultiTypeMap).
+        linkback: Whether to keep a pointer in the parent mixins to this
+            ovld so that updates can be propagated. (default: False)
     """
     ov = _find_overload(dispatch, **kwargs)
     return ov.dispatch(dispatch)
@@ -782,9 +872,11 @@ def rename_function(fn, newname):
         co.co_freevars,
         co.co_cellvars,
     )
-    return FunctionType(
+    new_fn = FunctionType(
         newcode, fn.__globals__, newname, fn.__defaults__, fn.__closure__
     )
+    new_fn.__annotations__ = fn.__annotations__
+    return new_fn
 
 
 __all__ = [
